const errorTypes = require("../errorTypes");
const constants = require("../constants");

const {createShortStack} = require("../stackparser");

describe("errorTypes.js", () => {

    function _makeMockShortStack(stack) {
         return createShortStack(stack)  
    }

    describe("OperationalError", () => {
        const testErrorMessage = "test message";
        const additionalErrorData = "extra data";
        const operationalError = new errorTypes.OperationalError(testErrorMessage, additionalErrorData);


        it("should be instance of OperationalError.", () => {
            expect(operationalError instanceof errorTypes.OperationalError).toBe(true);
        });

        it("should have name OperationalError.", () => {
            expect(operationalError.name).toBe("OperationalError");
        });

        it("should have property isOperational, and it should be true.", () => {
            expect(operationalError.isOperational).toBe(true);
        });

        it(`should have the message ${testErrorMessage}.`, () => {
            expect(operationalError.message).toBe(testErrorMessage);
        });

        it(`should have the data ${additionalErrorData}.`, () => {
            expect(operationalError.data).toEqual(JSON.stringify({stack: _makeMockShortStack(operationalError.stack), data: additionalErrorData,  }));
        });

        it("Should default error.includeBody to false", () => {
            expect(operationalError.includeBody).toBe(false);
        })

        const operationalErrorWithIncludeBody = new errorTypes.OperationalError(testErrorMessage, additionalErrorData, {includeBody: true});

        it("Should set error.includeBody to true, if it is passed in as true", () => {
            expect(operationalErrorWithIncludeBody.includeBody).toBe(true);
        })
    });

    describe("NonOperationalError", () => {
        const testErrorMessage = "test non operationalError";
        const additionalErrorData = "test additional data";
        const nonOperationalError = new errorTypes.NonOperationalError(testErrorMessage, additionalErrorData);

        it("should be instance of NonOperationalError", () => {
            expect(nonOperationalError instanceof errorTypes.NonOperationalError).toBe(true);

        });

        it("should have name NonOperationalError", () => {
            expect(nonOperationalError.name).toBe("NonOperationalError");
        });

        it("should have property isOperational, and it should be false.", () => {
            expect(nonOperationalError.isOperational).toBe(false);
        });

        it(`should have the message ${testErrorMessage}.`, () => {
            expect(nonOperationalError.message).toBe(testErrorMessage);
        });

        it(`should have the data ${additionalErrorData}.`, () => {
            expect(nonOperationalError.data).toEqual(JSON.stringify({ stack: _makeMockShortStack(nonOperationalError.stack), data: additionalErrorData }));
        });

        it("Should default error.includeBody to false", () => {
            expect(nonOperationalError.includeBody).toBe(false);
        });

        const nonOperationalErrorWithBody = new errorTypes.NonOperationalError(testErrorMessage, additionalErrorData, undefined, {includeBody: true});

        it("Should set error.includeBody to true if passed in as true", () => {
            expect(nonOperationalErrorWithBody.includeBody).toBe(true);
        })
    })

    describe("HttpError", () => {
        const testErrorMessage = "test http error";
        const testStatusCode = 401;
        const additionalErrorData = "test additional data";
        const httpError = new errorTypes.HttpError(testErrorMessage, testStatusCode, additionalErrorData);

        it("should be instance of HttpError.", () => {
            expect(httpError instanceof errorTypes.HttpError).toBe(true);
        });

        it("should have name HttpError.", () => {
            expect(httpError.name).toBe("HttpError");
        });

        it("should have property isOperational, and it should be true.", () => {
            expect(httpError.isOperational).toBe(true);
        });

        it(`should have the message ${testErrorMessage}.`, () => {
            expect(httpError.message).toBe(testErrorMessage);
        });

        it(`should have the data ${additionalErrorData}.`, () => {
            expect(httpError.data).toEqual(JSON.stringify({ stack: _makeMockShortStack(httpError.stack), data: additionalErrorData }));
        });

        it(`should have the status code ${testStatusCode}.`, () => {
            expect(httpError.statusCode).toBe(testStatusCode);
        });

        it("Should default error.includeBody to false", () => {
            expect(httpError.includeBody).toBe(false);
        });

        it("Should have error.sendToSlack as false if not passed in as true", () => {
            expect(httpError.sendToSlack).toBe(false);
        });

        it("Should have slackChannel as alerts_web if not passed in", () => {
            expect(httpError.slackChannel).toBe("alerts_web");
        });

        const httpErrorWithBody = new errorTypes.HttpError(testErrorMessage, testStatusCode, additionalErrorData, {includeBody: true});

        it("Should set error.includeBody to true if passed in as true", () => {
            expect(httpErrorWithBody.includeBody).toBe(true);
        });

        const testSlackChannel = "alerts_warnings";
        const httpErrorWithSlackOptions = new errorTypes.HttpError(testErrorMessage, testStatusCode, additionalErrorData, {slack: {send: true, channel: testSlackChannel}});

        it("Should have sentToSlack as true, if passed in as true in the options", () => {
            expect(httpErrorWithSlackOptions.sendToSlack).toBe(true);
        });

        it(`Should have slackChannel ${testSlackChannel} when passed in the options`, () => {
            expect(httpErrorWithSlackOptions.slackChannel).toBe(testSlackChannel);
        });
    })
})
